class RDSChecks:
    RDS_ENCRYPTION_AT_REST = "RDS instances MUST be configured for at-rest encryption"
    RDS_PUBLIC_ACCESSIBILITY = "RDS instances MUST NOT be configured to allow public accessibility"
    RDS_DEDICATED_VPC_SG = "RDS instances MUST have a dedicated VPC security group assigned to each separate RDS instance"
    RDS_PRIVATE_SUBNET_CHECK = "RDS instances MUST be deployed into private subnets designated for data"
    RDS_INGRESS_EGRESS_CHECK ="RDS instances MUST have least privilege ingress and egress VPC security group rules assigned"
    RDS_INSTANCE_CLUSTER_SNAPSHOTS_PUBLIC_ACCESSIBILITY = " RDS instance and cluster snapshots MUST NOT be publicly accessible"
    RDS_TAGS = "MUST be tagged in accordance with tagging standards"

class RdsMethodAssociations:
    RDS_ENCRYPTION_AT_REST = "check_encryption_at_rest"
    RDS_PUBLIC_ACCESSIBILITY = "check_rds_public_accessibility"
    RDS_DEDICATED_VPC_SG = "check_rds_dedicated_vpc_sg"
    RDS_PRIVATE_SUBNET_CHECK = "check_rds_private_subnet"
    RDS_INGRESS_EGRESS_CHECK = "check_rds_ingress_egress"
    RDS_INSTANCE_CLUSTER_SNAPSHOTS_PUBLIC_ACCESSIBILITY = "check_rds_instance_cluster_snapshot"
    RDS_TAGS = "check_rds_tags"

security_checks = {
    "RDS_ENCRYPTION_AT_REST": {
        "method_name": RdsMethodAssociations.RDS_ENCRYPTION_AT_REST,
        "check_description": RDSChecks.RDS_ENCRYPTION_AT_REST
    },
    "RDS_PUBLIC_ACCESSIBILITY": {
        "method_name": RdsMethodAssociations.RDS_PUBLIC_ACCESSIBILITY,
        "check_description": RDSChecks.RDS_PUBLIC_ACCESSIBILITY
    },
    "RDS_DEDICATED_VPC_SG": {
        "method_name": RdsMethodAssociations.RDS_DEDICATED_VPC_SG,
        "check_description": RDSChecks.RDS_DEDICATED_VPC_SG
    },
    "RDS_PRIVATE_SUBNET_CHECK": {
        "method_name": RdsMethodAssociations.RDS_PRIVATE_SUBNET_CHECK,
        "check_description": RDSChecks.RDS_PRIVATE_SUBNET_CHECK
    },
    "RDS_INGRESS_EGRESS_CHECK": {
        "method_name": RdsMethodAssociations.RDS_INGRESS_EGRESS_CHECK,
        "check_description": RDSChecks.RDS_INGRESS_EGRESS_CHECK
    },
    "RDS_INSTANCE_CLUSTER_SNAPSHOTS_PUBLIC_ACCESSIBILITY": {
        "method_name": RdsMethodAssociations.RDS_INSTANCE_CLUSTER_SNAPSHOTS_PUBLIC_ACCESSIBILITY,
        "check_description": RDSChecks.RDS_INSTANCE_CLUSTER_SNAPSHOTS_PUBLIC_ACCESSIBILITY
    },
    "RDS_TAGS": {
        "method_name": RdsMethodAssociations.RDS_TAGS,
        "check_description": RDSChecks.RDS_TAGS
    }
}

best_practices_checks = {
}

rds_dict = {
    "SecurityCheck": security_checks,
    "BestPractices": best_practices_checks,
    "AccountSecurityCheck": {},
    "AccountBestPractices": {}
}



import json
import boto3
import botocore
import dna_audit_tool.common.utils.helper as helper
from aws_services.emr import emr_constants as constants
from dna_audit_tool.common.constants import application_constants
from dna_audit_tool.common.utils.initialize_logger import logger

class EMR:
    """A class that checks the security settings of Amazon EMR Clusters."""

    def __init__(self):
        """
            Initializes an EMR,EC2 client object with the specified maximum number of retries in specified region.
        """
        try:
            configuration = helper.get_configuration()
            self.emr_client = boto3.client('emr',config=configuration)
            self.ec2_client = boto3.client('ec2',config=configuration)

        except Exception as ex:
            logger.error("Error occurred while initializing EMR and EC2 client objects: %s", str(ex))
            raise ex
       

    def list_resources(self):
        """
        Returns a list of all Amazon EMR clusters associated with the EMR client which are in RUNNING state.

        Returns:
        - A list of Active Amazon EMR cluster IDs.

        Raises:
        - botocore.exceptions.ClientError: if there is an error communicating with AWS.
        """
        try:
            terminating_clusters = ["TERMINATING", "TERMINATED", "TERMINATED_WITH_ERRORS"]
            clusters = []
            marker = ''
            while True:
                if marker:
                    response = self.emr_client.list_clusters(Marker=marker)
                else:
                    response = self.emr_client.list_clusters()
                for cluster in response['Clusters']:
                    if cluster['Status']['State'] not in terminating_clusters:
                        clusters.append(cluster['Id'])
                if 'Marker' in response:
                    marker = response['Marker']
                else:
                    break
            return clusters
        except botocore.exceptions.ClientError as ex:
            logger.error("Error occurred when listing Amazon EMR clusters: %s", str(ex))
            raise ex


    def check_emr_encryption_at_rest(self, cluster_id):
        """
        Check if encryption at rest is enabled for a given EMR cluster.

        Args:
            cluster_id (str): ID of the EMR cluster to check.

        Returns:
            service: This method returns the status of the check.

        Raises:
            Exception: If any error occurs during the check.

        """
        logger.info("Checking for encryption at rest for EMR cluster %s", cluster_id)

        try:
            response = self.emr_client.describe_cluster(ClusterId=cluster_id)

            if 'SecurityConfiguration' in response['Cluster'] and response['Cluster']['SecurityConfiguration']:
                security_config = self.emr_client.describe_security_configuration(
                    Name=response['Cluster']['SecurityConfiguration']
                )
                if 'EncryptionConfiguration' in security_config['SecurityConfiguration']:
                    try:
                        security_conf = json.loads(security_config['SecurityConfiguration'])
                    except:
                        security_conf = eval(security_config['SecurityConfiguration'])
                    encryption_config = security_conf['EncryptionConfiguration']
                    if 'EnableAtRestEncryption' in encryption_config and encryption_config['EnableAtRestEncryption']:
                        check_result = application_constants.ResultStatus.PASSED
                    else:
                        check_result = application_constants.ResultStatus.FAILED
                else:
                    check_result = application_constants.ResultStatus.FAILED
            else:
                check_result = application_constants.ResultStatus.FAILED

        except self.emr_client.exceptions.ClientError as ex:
            error_code = ex.response['Error']['Code']
            if error_code == 'ClusterNotFound':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error("Error checking encryption at rest for EMR cluster %s: %s", cluster_id, str(ex))
                check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("Error occurred during check for encryption at rest for EMR cluster %s: %s", cluster_id, str(ex))
            raise ex

        logger.info("Completed checking for encryption at rest for EMR cluster %s", cluster_id)
        return check_result


    def check_least_privilege_sg(self, cluster_id):
        """
        Check if least privilege ingress and egress security group rules are assigned to the given EMR cluster.

        Args:
            cluster_id (str): ID of the EMR cluster to check.

        Returns:
            dict: A dictionary containing the status of the check.

        Raises:
            Exception: If any error occurs during the check.
        """
        logger.info("Checking for least privilege SG rules for EMR cluster %s", cluster_id)

        try:
            response = self.emr_client.describe_cluster(ClusterId=cluster_id)

            if 'Ec2InstanceAttributes' in response['Cluster'] and 'EmrManagedMasterSecurityGroup' in response['Cluster']['Ec2InstanceAttributes'] and 'EmrManagedSlaveSecurityGroup' in response['Cluster']['Ec2InstanceAttributes']:
                master_sg = response['Cluster']['Ec2InstanceAttributes']['EmrManagedMasterSecurityGroup']
                slave_sg = response['Cluster']['Ec2InstanceAttributes']['EmrManagedSlaveSecurityGroup']
                master_sg_rules_Ids=response['Cluster']['Ec2InstanceAttributes']['AdditionalMasterSecurityGroups']
                slave_sg_rules_Ids=response['Cluster']['Ec2InstanceAttributes']['AdditionalSlaveSecurityGroups']
                sg_ids_all=[]
                sg_ids_all=master_sg_rules_Ids+slave_sg_rules_Ids
                sg_ids_all.append(master_sg); sg_ids_all.append(slave_sg)
                sg_rules = self.ec2_client.describe_security_groups(GroupIds=sg_ids_all)['SecurityGroups']
                sg_rules_new=[]
                for sg in sg_rules:
                    sg_rules_new+=sg['IpPermissions']
                if helper.is_least_privilege_sg(sg_rules_new):
                    check_result = application_constants.ResultStatus.PASSED
                else:
                    check_result = application_constants.ResultStatus.FAILED
            else:
                check_result = application_constants.ResultStatus.UNKNOWN

        except self.emr_client.exceptions.ClientError as ex:
            error_code = ex.response['Error']['Code']
            if error_code == 'ClusterNotFound':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error("Error checking for least privilege SG rules for EMR cluster %s: %s", cluster_id, str(ex))
                check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("Error occurred during check for least privilege SG rules for EMR cluster %s: %s", cluster_id, str(ex))
            raise ex

        logger.info("Completed checking for least privilege SG rules for EMR cluster %s", cluster_id)
        return check_result


    def check_emr_cluster_tags(self, cluster_id, required_tags=None):
        """
        Checks if the specified EMR cluster has the required tags.

        Args:
            cluster_id (str): The ID of the EMR cluster.

        Returns:
            dict: A dictionary containing the result of the check.
        """
        try:
            if required_tags is None:
                required_tags = application_constants.Generic.REQUIRED_TAGS

            if not required_tags:
                check_result = application_constants.ResultStatus.PASSED
            else:
                response = self.emr_client.describe_cluster(ClusterId=cluster_id)
                tags = response['Cluster']['Tags']
                missing_tags = [tag for tag in required_tags if tag.split(":")[0] not in [t["Key"] for t in tags]]
                check_result = application_constants.ResultStatus.PASSED if not any(missing_tags) else application_constants.ResultStatus.FAILED

            logger.info("Completed checking the tags for EMR cluster %s", cluster_id)
            return check_result

        except botocore.exceptions.ClientError as ex:
            if ex.response['Error']['Code'] == 'ResourceNotFoundException':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error("An error occurred while checking the tags for EMR cluster %s: %s", cluster_id, str(ex))
                check_result = application_constants.ResultStatus.UNKNOWN

            logger.info("Completed checking the tags for EMR cluster %s", cluster_id)
            return check_result

        except Exception as ex:
            logger.exception("An error occurred while checking the tags for EMR cluster %s: %s", cluster_id, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN

            logger.info("Completed checking the tags for EMR cluster %s", cluster_id)
            return check_result



    def check_emr_cluster_in_private_subnet(self, cluster_id):
        """
        Check if an EMR cluster is in a private subnet.

        Args:
            cluster_id (str): The ID of the EMR cluster to check.

        Returns:
            bool: True if the EMR cluster is in a private subnet, False otherwise.

        Raises:
            Exception: If any error occurs during the check.
        """

        try:
            response = self.emr_client.describe_cluster(ClusterId=cluster_id)
            subnet_id = response['Cluster']['Ec2InstanceAttributes']['Ec2SubnetId']
            is_public = helper.check_subnet_has_igw(subnet_id)

            check_result = application_constants.ResultStatus.FAILED if is_public else application_constants.ResultStatus.PASSED

            logger.info("Completed checking EMR cluster for private subnet check %s", cluster_id)
            return check_result

        except Exception as ex:
            logger.exception("An error occurred while checking EMR cluster for private subnet check %s: %s", cluster_id, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN

            logger.info("Completed checking EMR cluster for private subnet check %s", cluster_id)
            return check_result



import boto3
import botocore
import dna_audit_tool.common.utils.helper as helper
from aws_services.fargate import fargate_constants as constants
from dna_audit_tool.common.constants import application_constants
from dna_audit_tool.common.utils.initialize_logger import logger


class Fargate:
    """A class that checks the security settings of Fargate Services."""

    def __init__(self):
        """
            Initializes an ECS,EC2 client object with the specified maximum number of retries in specified region.
        """
        try:
            configuration = helper.get_configuration()
            self.fargate_client = boto3.client('ecs',config=configuration)
            self.ec2_client = boto3.client('ec2',config=configuration)
        except Exception as ex:
            logger.error("Error occurred while initializing ECS and EC2 client objects: %s", str(ex))
            raise ex
        


    def list_resources(self):
        """
        Returns a list of all Amazon ECS Fargate services .

        Returns:
            - A list of ECS Fargate Service Arns.

            Raises:
            - botocore.exceptions.ClientError: if there is an error communicating with AWS.
        """
        try:
            clustersArns_list=[]
            list_resources_final = []
            paginator_cluster = self.fargate_client.get_paginator('list_clusters')
            for page in paginator_cluster.paginate():
                clustersArns_list+=page['clusterArns']
            for cluster_arn in clustersArns_list:
                paginator_services=self.fargate_client.get_paginator('list_services')
                task_params={'cluster': cluster_arn,'launchType': 'FARGATE'}
                for page in paginator_services.paginate(**task_params):
                    for serviceArn in page['serviceArns']:
                        services_status = self.fargate_client.describe_services(cluster=cluster_arn, services=[serviceArn])['services'][0]['status']
                        if services_status=='ACTIVE':
                            list_resources_final.append(serviceArn)
            return list_resources_final
        except botocore.exceptions.ClientError as ex:
            logger.error(
                "Error occurred when listing fargate services: %s", str(ex))
            raise ex


    def helper_find_cluster(self, service_Arn):
        """
        Returns cluterArn if the fargate serviceArn is passed in parameter. 

        Args:
            service_Arn (str): The serviceArn of the fargate service.

        Returns:
            clusterArn of the service.

        Raises:
            Exception: If any error occurs during the check.
        """
        try:
            clustersArns_list=[]
            paginator_cluster = self.fargate_client.get_paginator('list_clusters')
            for page in paginator_cluster.paginate():
                clustersArns_list+=page['clusterArns']
            list_resources_final = []
            for cluster_arn in clustersArns_list:
                services = self.fargate_client.list_services(cluster=cluster_arn, launchType='FARGATE')['serviceArns']
                for service in services:
                    list_resources_final.append(service)
                if service_Arn in list_resources_final:
                    return cluster_arn
        except botocore.exceptions.ClientError as ex:
            logger.error(
                "Error occurred when listing fargate services: %s", str(ex))
            raise ex


    def check_public_access(self, fargate_service_Arn):
        """
        checks if the given service is publicly accessible or not.

        Args:
            fargate_service_Arn(str): .

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        Raises:
            Exception: If any error occurs during the check.

        """
        logger.info("fetching the service details for %s",
                    fargate_service_Arn)
        try:
            clusterArn = self.helper_find_cluster(fargate_service_Arn)
            services_details = self.fargate_client.describe_services(cluster=clusterArn, services=[fargate_service_Arn])
            if 'networkConfiguration' not in services_details['services'][0]:
                check_result=application_constants.ResultStatus.UNKNOWN

            elif services_details['services'][0]['networkConfiguration']['awsvpcConfiguration']['assignPublicIp'] == 'ENABLED':
                check_result = application_constants.ResultStatus.FAILED
            else:
                check_result = application_constants.ResultStatus.PASSED
        except self.fargate_client.exceptions.ClientError as ex:
            logger.error(
                "error while checking public access for the service %s: %s", fargate_service_Arn, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error(
                "exception while checking public access for the service %s: %s", fargate_service_Arn, str(ex))
            raise ex
        logger.info(
            "Completed checking the public access for the service %s", fargate_service_Arn)
        return check_result


    def check_if_in_private_subnet(self, fargate_service_Arn):
        """
        checks if the service is in private subnet.

        Args:
            fargate_service_Arn(str): .

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        Raises:
            Exception: If any error occurs during the check.

        """
        logger.info("fetching the service details for %s",
                    fargate_service_Arn)
        try:
            clusterArn = self.helper_find_cluster(fargate_service_Arn)
            services_details = self.fargate_client.describe_services(cluster=clusterArn, services=[fargate_service_Arn])
            if 'networkConfiguration' not in services_details['services'][0]:
                check_result=application_constants.ResultStatus.UNKNOWN
            else:
                subnets = services_details['services'][0]['networkConfiguration']['awsvpcConfiguration']['subnets']
                for subnet in subnets:
                    is_public = helper.check_subnet_has_igw(subnet)
                    if is_public:
                        check_result=application_constants.ResultStatus.FAILED
                        break
                    else:
                        check_result = application_constants.ResultStatus.PASSED
        except self.fargate_client.exceptions.ClientError as ex:
            logger.error(
                "error while checking if service in private subnet for %s: %s", fargate_service_Arn, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error(
                "exception while checking if service in private subnet for %s: %s", fargate_service_Arn, str(ex))
            raise ex
        logger.info(
            "Completed checking if service in private subnet   %s", fargate_service_Arn)
        return check_result


    def check_ip_permissions(self, fargate_service_Arn):
        """
        checks if the service has inbound traffic from everywhere.

        Args:
            fargate_service_Arn(str): .

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        Raises:
            Exception: If any error occurs during the check.

        """
        logger.info("fetching the service details for %s",fargate_service_Arn)
        try:
            clusterArn = self.helper_find_cluster(fargate_service_Arn)
            services_details = self.fargate_client.describe_services(cluster=clusterArn, services=[fargate_service_Arn])
            if 'networkConfiguration' not in services_details['services'][0]:
                check_result = application_constants.ResultStatus.UNKNOWN
            else:
                security_groups = services_details['services'][0]['networkConfiguration']['awsvpcConfiguration']['securityGroups']
                for sg in security_groups:
                    sg_details = self.ec2_client.describe_security_groups(GroupIds=[sg])['SecurityGroups'][0]['IpPermissions']
                    if helper.is_least_privilege_sg(sg_details):
                        check_result = application_constants.ResultStatus.PASSED
                    else:
                        check_result = application_constants.ResultStatus.FAILED
        except self.fargate_client.exceptions.ClientError as ex:
            logger.error(
                "error while checking ip permissions of the service  %s: %s", fargate_service_Arn, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error(
                "exception while checking ip permissions of the service %s: %s", fargate_service_Arn, str(ex))
            raise ex
        logger.info(
            "Completed fetching ip permissions for fargate service %s", fargate_service_Arn)
        return check_result


    def check_fargate_tags(self, fargate_service_Arn, required_tags=None):
        """
        Checks if the specified Fargate has the required tags.

        Args:
            fargate_service_Arn (str): The Arn of the Fargate service.

        Returns:
            dict: A dictionary containing the result of the check.
        """
        try:
            if required_tags is None:
                required_tags = application_constants.Generic.REQUIRED_TAGS

            if not required_tags:
                check_result = application_constants.ResultStatus.PASSED
            else:
                clusterArn = self.helper_find_cluster(fargate_service_Arn)
                tags = self.fargate_client.list_tags_for_resource( resourceArn=fargate_service_Arn)['tags']
                missing_tags = [tag for tag in required_tags if tag.split(":")[0] not in [t["key"] for t in tags]]
                check_result = application_constants.ResultStatus.PASSED if not any(missing_tags) else application_constants.ResultStatus.FAILED
        except botocore.exceptions.ClientError as ex:
            if ex.response['Error']['Code'] == 'NoSuchTagSet':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error(
                    f"An error occurred while checking the tags for fargate service {fargate_service_Arn}: {str(ex)}")
                check_result = application_constants.ResultStatus.UNKNOWN

        except Exception as ex:
            logger.exception(
                f"An error occurred while checking the tags for fargate service {fargate_service_Arn}: {str(ex)}")
            check_result = application_constants.ResultStatus.UNKNOWN

        logger.info(
            f"Completed checking the tags for fargate service {fargate_service_Arn}")
        return check_result


class LambdaChecks:
    LAMBDA_REQUIRED_TAGS = "MUST be tagged in accordance with tagging standards"

class LambdaMethodAssociations:
    LAMBDA_REQUIRED_TAGS = "check_lambda_function_tags"

security_checks = {
    "LAMBDA_REQUIRED_TAGS": {
        "method_name": LambdaMethodAssociations.LAMBDA_REQUIRED_TAGS,
        "check_description": LambdaChecks.LAMBDA_REQUIRED_TAGS
    }
}

best_practices_checks = {
}

check_dict = {
    "SecurityCheck": security_checks,
    "BestPractices": best_practices_checks,
    "AccountSecurityCheck": {},
    "AccountBestPractices": {}
}



class RedshiftChecks:
    REDSHIFT_CHECK_AT_REST_ENCRYPTION = "Redshift clusters MUST be configured for at-rest encryption using a KMS Key"
    REDSHIFT_CHECK_PUBLIC_ACCESSIBILITY = "Redshift clusters MUST NOT be configured to allow public accessibility"
    REDSHIFT_CHECK_PRIVATE_SUBNET = "Redshift clusters MUST be deployed into a managed VPC and assigned to a subnet " \
                                    "group containing private subnets designated for data"
    REDSHIFT_CHECK_DEDICATED_SECURITY_GROUP = "Redshift clusters MUST have a dedicated VPC security group " \
                                              "assigned to each separate Redshift cluster"
    REDSHIFT_CHECK_SG_LEAST_INGRESS_EGRESS_RULES = "Redshift clusters MUST have least privilege ingress and egress " \
                                                   "VPC security group rules assigned"
    REDSHIFT_CHECK_TAGS = "MUST be tagged in accordance with tagging standards"


class RedshiftMethodAssociations:
    REDSHIFT_CHECK_AT_REST_ENCRYPTION = "check_encryption_at_rest"
    REDSHIFT_CHECK_PUBLIC_ACCESSIBILITY = "check_redshift_public_accessibility"
    REDSHIFT_CHECK_PRIVATE_SUBNET = "check_redshift_private_subnet"
    REDSHIFT_CHECK_DEDICATED_SECURITY_GROUP = "check_redshift_dedicated_security_group"
    REDSHIFT_CHECK_SG_LEAST_INGRESS_EGRESS_RULES = "check_redshift_ingress_egress"
    REDSHIFT_CHECK_TAGS = "check_redshift_tags"


security_checks = {
    "REDSHIFT_CHECK_AT_REST_ENCRYPTION": {
        "method_name": RedshiftMethodAssociations.REDSHIFT_CHECK_AT_REST_ENCRYPTION,
        "check_description": RedshiftChecks.REDSHIFT_CHECK_AT_REST_ENCRYPTION
    },
    "REDSHIFT_CHECK_PUBLIC_ACCESSIBILITY": {
        "method_name": RedshiftMethodAssociations.REDSHIFT_CHECK_PUBLIC_ACCESSIBILITY,
        "check_description": RedshiftChecks.REDSHIFT_CHECK_PUBLIC_ACCESSIBILITY
    },
    "REDSHIFT_CHECK_PRIVATE_SUBNET": {
        "method_name": RedshiftMethodAssociations.REDSHIFT_CHECK_PRIVATE_SUBNET,
        "check_description": RedshiftChecks.REDSHIFT_CHECK_PRIVATE_SUBNET
    },
    "REDSHIFT_CHECK_DEDICATED_SECURITY_GROUP": {
        "method_name": RedshiftMethodAssociations.REDSHIFT_CHECK_DEDICATED_SECURITY_GROUP,
        "check_description": RedshiftChecks.REDSHIFT_CHECK_DEDICATED_SECURITY_GROUP
    },
    "REDSHIFT_CHECK_SG_LEAST_INGRESS_EGRESS_RULES": {
        "method_name": RedshiftMethodAssociations.REDSHIFT_CHECK_SG_LEAST_INGRESS_EGRESS_RULES,
        "check_description": RedshiftChecks.REDSHIFT_CHECK_SG_LEAST_INGRESS_EGRESS_RULES
    },
    "REDSHIFT_CHECK_TAGS": {
        "method_name": RedshiftMethodAssociations.REDSHIFT_CHECK_TAGS,
        "check_description": RedshiftChecks.REDSHIFT_CHECK_TAGS
    }
}

best_practices_checks = {
}

redshift_dict = {
    "SecurityCheck": security_checks,
    "BestPractices": best_practices_checks,
    "AccountSecurityCheck": {},
    "AccountBestPractices": {}
}



class SQSChecks:
    SQS_CHECK_AT_REST_ENCRYPTION = "SQS queues MUST be configured for at-rest encryption using a KMS Key"
    SQS_MANAGED_ACCOUNT_ACCESS = "SQS queues MUST be configured for least privilege access from accounts"
    SQS_PUBLIC_POLICY_ACCESS = "SQS queues MUST NOT allow anonymous access"
    SQS_CHECK_IN_TRANSIT_ENCRYPTION = "SQS queues SHOULD be configured for encryption in transit using SSL/TLS whenever possible"
    SQS_CHECK_VPC_ENDPOINT = "SQS queues SHOULD use VPC endpoints whenever possible"
    SQS_CHECK_UNUSED_QUEUES = "Unused SQS queues SHOULD be deleted"
    SQS_CHECK_TAGS = "SQS queues MUST be tagged in accordance with tagging standards"

class SQSMethodAssociations:
    SQS_CHECK_AT_REST_ENCRYPTION = "check_encryption_at_rest"
    SQS_MANAGED_ACCOUNT_ACCESS = "check_sqs_policy"
    SQS_PUBLIC_POLICY_ACCESS = "check_sqs_public_policy"
    SQS_CHECK_IN_TRANSIT_ENCRYPTION = "check_in_transit_encryption"
    SQS_CHECK_VPC_ENDPOINT = "check_sqs_vpc_endpoint"
    SQS_CHECK_UNUSED_QUEUES = "check_unused_queues"
    SQS_CHECK_TAGS = "check_queue_tags"

security_checks = {
    "SQS_CHECK_AT_REST_ENCRYPTION": {
        "method_name": SQSMethodAssociations.SQS_CHECK_AT_REST_ENCRYPTION,
        "check_description": SQSChecks.SQS_CHECK_AT_REST_ENCRYPTION
    },
    "SQS_PUBLIC_POLICY_ACCESS": {
        "method_name": SQSMethodAssociations.SQS_PUBLIC_POLICY_ACCESS,
        "check_description": SQSChecks.SQS_PUBLIC_POLICY_ACCESS
    },
    "SQS_CHECK_TAGS": {
        "method_name": SQSMethodAssociations.SQS_CHECK_TAGS,
        "check_description": SQSChecks.SQS_CHECK_TAGS
    }
}

best_practices_checks = {
    "SQS_CHECK_IN_TRANSIT_ENCRYPTION": {
        "method_name": SQSMethodAssociations.SQS_CHECK_IN_TRANSIT_ENCRYPTION,
        "check_description": SQSChecks.SQS_CHECK_IN_TRANSIT_ENCRYPTION
    },
    "SQS_CHECK_VPC_ENDPOINT": {
        "method_name": SQSMethodAssociations.SQS_CHECK_VPC_ENDPOINT,
        "check_description": SQSChecks.SQS_CHECK_VPC_ENDPOINT
    },
    # "SQS_CHECK_UNUSED_QUEUES": {
    #     "method_name": SQSMethodAssociations.SQS_CHECK_UNUSED_QUEUES,
    #     "check_description": SQSChecks.SQS_CHECK_UNUSED_QUEUES
    # }
}

check_dict = {
    "SecurityCheck": security_checks,
    "BestPractices": best_practices_checks,
    "AccountSecurityCheck": {},
    "AccountBestPractices": {}
}



import os
import json
from dna_audit_tool.common.constants import application_constants as constants
from dna_audit_tool.common.utils import helper
from dna_audit_tool.common.utils import instance_initializer
from dna_audit_tool.common.utils.initialize_logger import logger


def check_config_exists():
    """Checks if the configuration file exists.

    Returns:
        bool: True if the file exists, False otherwise.
    """
    config_path=os.environ.get('Config_Path')
    try:
        return os.path.exists(config_path)
    except Exception as ex:
        logger.error("Error while checking if the configuration file exists: %s", ex)
        return False


def get_or_create_config():
    """Retrieves an AWS resource list and creates a new configuration file or loads the existing one.

    Returns:
        dict: A dictionary containing the AWS resource configurations.
    """
    try:
        config_path=os.environ.get('Config_Path')
        config_exists = check_config_exists()
        config_changes={}
        if config_exists:
            with open(config_path, encoding='utf-8') as f:
                config_data = json.load(f)
            if config_data['region_name'] == os.environ.get('AWS_DEFAULT_REGION'):
                for key in config_data:
                    changes=[]
                    if key not in ('account_tags','region_name','resource'):
                        for resource in config_data[key]:
                            for check in resource['security_requirement_checks']:
                                if check['status']=='Disabled':
                                    changes.append({
                                        "resource_name":resource['resource_name'],
                                        "check_name":check['check_name'],
                                        "status":check['status']

                                    })
                            for check in resource['security_best_practices_checks']:
                                if check['status']=='Disabled':
                                    changes.append({
                                        "resource_name":resource['resource_name'],
                                        "check_name":check['check_name'],
                                        "status":check['status']

                                    })
                        config_changes[key]=changes
            if 'account_tags' in config_data:
                config_changes["account_tags"] = config_data["account_tags"]
            logger.info("Loaded existing configuration file and saved the disabled checks if existed.")

        logger.info("Started creating new configuration file.")
        output = {}
        for resource_name in instance_initializer.initialize_members():
            list_resources_method = getattr(instance_initializer.initialize_members()[resource_name], 'list_resources')
            output[resource_name] = list_resources_method()
        config_data = helper.create_config_file(output,config_changes)
        helper.store_config_file(config_data)
    except Exception as ex:
        logger.error("Error while getting/creating the configuration file: %s", ex)
        raise
    return config_data


def validate_security_checks(resource, check_key, overall_result, root_key, check_name, account_checks):
    """ Function to validate security checks
    Args:
        resource (dict): base resource dict
        check_key (string): check key
        overall_result (dict): overall result
        root_key (string): root key (ie security requirement, best_practices)
        check_name (string): check name
        account_checks (string): account checks
    Raises:
        ex: Error while validating the security requirements
    """
    try:
        for check in resource[check_key]:

            overall_check_dict = constants.overall_check_dict[root_key][check_name]
            overall_check_dict.update(constants.overall_check_dict[root_key][account_checks])
            check_description = overall_check_dict[check['check_name']]['check_description']
            region=os.environ.get('region')
            if check['status'] == "Enabled":
                method_name = overall_check_dict[check['check_name']]['method_name']
                check_result = getattr(instance_initializer.initialize_members()[root_key], method_name)(resource['resource_name'])
                if check_result:
                    overall_result[resource['resource_name']].append(
                    {
                        "check_name": check_description,
                        "status": check_result,
                        "type": check_name
                    })
            else:
                overall_result[resource['resource_name']].append(
                {
                    "check_name": check_description,
                    "status": constants.ResultStatus.DISABLED,
                    "type": check_name
                })
    except Exception as ex:
        logger.error("Error while validating the security requirements: %s", ex)
        raise ex


def evaluate_standards(config_data):
    """Evaluates the security requirements for each AWS resources in the configuration data.

    Args:
        config_data (dict): A dictionary containing the AWS resources configurations.

    Returns:
        dict: A dictionary containing the results of the security requirement evaluations.
    """
    service_info = {}
    try:
        for key in config_data:
            if key not in ("account_tags","region_name"):
                overall_result = {}
                for resource in config_data[key]:
                    overall_result[resource['resource_name']] = []
                    validate_security_checks(resource, 'security_requirement_checks', overall_result, key, constants.Type.SECURITY_CHECK, constants.Type.ACC_SECURITY_CHECK)
                    validate_security_checks(resource, 'security_best_practices_checks', overall_result, key, constants.Type.BEST_PRACTICES, constants.Type.ACC_BEST_PRACTICES)
                    logger.info("Evaluation completed for %s.", resource['resource_name'])
                service_info[key] = overall_result

    except Exception as ex:
        logger.error("Error while evaluating the security requirements: %s", ex)
        raise ex
    return service_info



class DynamoDbChecks:
    DYNAMODB_CHECK_AT_REST_ENCRYPTION = "DynamoDB tables MUST be configured for at-rest encryption using a KMS Key"
    DYNAMODB_CHECK_AUTOMATIC_BACKUP = "DynamoDB tables MUST be configured for automatic backup"
    DYNAMODB_CHECK_DELETE_PROTECTION = "DynamoDB tables MUST be enabled for DeleteProtection"
    DYNAMODB_CHECK_TAGS = "MUST be tagged in accordance with tagging standards"


class DynamoDbMethodAssociations:
    DYNAMODB_CHECK_AT_REST_ENCRYPTION = "check_encryption_at_rest"
    DYNAMODB_CHECK_AUTOMATIC_BACKUP = "check_automatic_backups"
    DYNAMODB_CHECK_DELETE_PROTECTION = "check_delete_protection"
    DYNAMODB_CHECK_TAGS = "check_dynamodb_tags"


security_checks = {
    "DYNAMODB_CHECK_AT_REST_ENCRYPTION": {
        "method_name": DynamoDbMethodAssociations.DYNAMODB_CHECK_AT_REST_ENCRYPTION,
        "check_description": DynamoDbChecks.DYNAMODB_CHECK_AT_REST_ENCRYPTION
    },
    "DYNAMODB_CHECK_AUTOMATIC_BACKUP": {
        "method_name": DynamoDbMethodAssociations.DYNAMODB_CHECK_AUTOMATIC_BACKUP,
        "check_description": DynamoDbChecks.DYNAMODB_CHECK_AUTOMATIC_BACKUP
    },
    "DYNAMODB_CHECK_DELETE_PROTECTION": {
        "method_name": DynamoDbMethodAssociations.DYNAMODB_CHECK_DELETE_PROTECTION,
        "check_description": DynamoDbChecks.DYNAMODB_CHECK_DELETE_PROTECTION
    },
    "DYNAMODB_CHECK_TAGS": {
        "method_name": DynamoDbMethodAssociations.DYNAMODB_CHECK_TAGS,
        "check_description": DynamoDbChecks.DYNAMODB_CHECK_TAGS
    }
}

best_practices_checks = {
}

dynamodb_dict = {
    "SecurityCheck": security_checks,
    "BestPractices": best_practices_checks,
    "AccountSecurityCheck": {},
    "AccountBestPractices": {}
}



import collections

import boto3
import botocore

from dna_audit_tool.common.constants import application_constants
from dna_audit_tool.common.utils import helper
from dna_audit_tool.common.utils.initialize_logger import logger


class DynamoDB:
    """A class that checks the security settings of Amazon DynamoDB Tables."""

    def __init__(self):
        """
            Initializes a DynamoDb client object with the specified maximum number of retries in specified region.
        """
        try:
            configuration = helper.get_configuration()
            self.service = collections.defaultdict(dict)
            self.dynamodb_client = boto3.client('dynamodb', config=configuration)
            self.resource_list = []
        except Exception as ex:
            logger.error("Error occurred while initializing dynamoDb client objects: %s", str(ex))
            raise ex

    def list_resources(self):
        """
        Returns a list of all dynamoDb tables associated with the boto client.

        Parameters:
        - None

        Returns:
        - A list of DynamoDB table names.

        Raises:
        - botocore.exceptions.ClientError: if there is an error communicating with AWS.
        """
        try:
            paginator = self.dynamodb_client.get_paginator('list_tables')
            for page in paginator.paginate(PaginationConfig={'PageSize': 20}):
                self.resource_list += page['TableNames']
            return self.resource_list

        except botocore.exceptions.ClientError as ex:
            logger.error("Error occurred when listing DynamoDB resources: %s", str(ex))
            raise

    def check_encryption_at_rest(self, table_name):
        """
        Check if encryption at rest is enabled for a given dynamodb table .

        Args:
            table_name (str): Table name of the dynamodb table to check.

        Returns:
            Returns the Status of the Validation Check.

        Raises:
            ClientError: if there is an error communicating with AWS.
            ResourceNotFoundException: if the requested table not found
            InternalServerError : if the server responded with internal server error
            Exception: If any error occurs during the check.

        """
        logger.info("gathering the encryption settings for %s", table_name)
        try:
            table = self.dynamodb_client.describe_table(TableName=table_name)['Table']
            if 'SSEDescription' in table and 'Status' in table['SSEDescription'] and table['SSEDescription'][
                'Status'] == 'ENABLED':
                check_result = application_constants.ResultStatus.PASSED
            else:
                check_result = application_constants.ResultStatus.FAILED
        except (
                self.dynamodb_client.exceptions.ResourceNotFoundException,
                self.dynamodb_client.exceptions.InternalServerError,
                self.dynamodb_client.exceptions.ClientError) as ex:
            logger.error("error while gathering the encryption settings for %s: %s", table_name, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("error while gathering the encryption settings for %s: %s", table_name, str(ex))
            raise ex

        logger.info("Completed fetching encryption settings for dynamodb table : %s", table_name)
        return check_result

    def check_automatic_backups(self, table_name):
        """
        Check if automatic backup is enabled for a given dynamodb table .

        Args:
            table_name (str): Table name of the dynamodb table to check.

        Returns:
            Returns the Status of the Validation Check.

        Raises:
            ClientError: if there is an error communicating with AWS.
            TableNotFoundException: if the requested table not found
            InternalServerError : if the server responded with internal server error
            Exception: If any error occurs during the check.

        """
        logger.info("gathering the backup settings for %s", table_name)
        try:
            response = self.dynamodb_client.describe_continuous_backups(TableName=table_name)['ContinuousBackupsDescription']
            if 'ContinuousBackupsStatus' in response and  response['ContinuousBackupsStatus'] == 'ENABLED':
                check_result = application_constants.ResultStatus.PASSED
            else:
                check_result = application_constants.ResultStatus.FAILED
        except (
                self.dynamodb_client.exceptions.TableNotFoundException,
                self.dynamodb_client.exceptions.InternalServerError,
                self.dynamodb_client.exceptions.ClientError) as ex:
            logger.error("error while gathering the backup settings for %s: %s", table_name, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("error while gathering the backup settings for %s: %s", table_name, str(ex))
            raise ex

        logger.info("Completed fetching backup settings for dynamodb table : %s", table_name)
        return check_result

    def check_delete_protection(self, table_name):
        """
        Check if delete protection is enabled for a given dynamodb table .

        Args:
            table_name (str): Table name of the dynamodb table to check.

        Returns:
            Returns the Status of the Validation Check.

        Raises:
            ClientError: if there is an error communicating with AWS.
            ResourceNotFoundException: if the requested table not found
            InternalServerError : if the server responded with internal server error
            Exception: If any error occurs during the check.

        """
        logger.info("gathering the delete protection settings for %s", table_name)
        try:
            table = self.dynamodb_client.describe_table(TableName=table_name)['Table']
            if 'DeletionProtectionEnabled' in table and table['DeletionProtectionEnabled'] == 'True':
                check_result = application_constants.ResultStatus.PASSED
            else:
                check_result = application_constants.ResultStatus.FAILED
        except (
                self.dynamodb_client.exceptions.ResourceNotFoundException,
                self.dynamodb_client.exceptions.InternalServerError,
                self.dynamodb_client.exceptions.ClientError) as ex:
            logger.error("error while gathering the delete protection  settings for %s: %s", table_name, str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN
        except Exception as ex:
            logger.error("error while gathering the delete protection  settings for %s: %s", table_name, str(ex))
            raise ex

        logger.info("Completed fetching delete protection settings for dynamodb table : %s", table_name)
        return check_result

    def check_dynamodb_tags(self, table_name, required_tags=None):
        """
        Checks if the specified DynamoDB table has the required tags.

        Args:
             table_name (str): Table name of the DynamoDb Table to check.

        Returns:
            Returns the Status of the Validation Check.
        """

        if required_tags is None:
            required_tags = application_constants.Generic.REQUIRED_TAGS

        if not required_tags:
            check_result = application_constants.ResultStatus.PASSED
        else:
            logger.info("Checking the tags for %s", table_name)
            try:
                table = self.dynamodb_client.describe_table(TableName=table_name)['Table']

                paginator = self.dynamodb_client.get_paginator('list_tags_of_resource')
                pages = paginator.paginate(ResourceArn=table['TableArn'])
                tags = []
                for page in pages:
                    tags += page['Tags']

                missing_tags = [tag for tag in required_tags if tag not in [t['Key'] for t in tags]]
                check_result = application_constants.ResultStatus.PASSED if not any(
                    missing_tags) else application_constants.ResultStatus.FAILED
            except Exception as ex:
                logger.exception(f"An error occurred while checking the tags for {table_name}: {str(ex)}")
                check_result = application_constants.ResultStatus.UNKNOWN

        logger.info(f"Completed checking the tags for resource : {table_name}")
        return check_result



import collections
import boto3
import botocore
import dna_audit_tool.common.utils.helper as helper

from dna_audit_tool.common.constants import application_constants
from aws_services.glue import glue_constants as constants
from dna_audit_tool.common.utils.initialize_logger import logger


class Glue:
    """A class that checks the security settings of Glue."""

    def __init__(self):
        """
            Initializes an glue, iam client object with the specified maximum number of retries in specified region.
        """
        try:
            configuration = helper.get_configuration()
            self.glue_client = boto3.client('glue',config = configuration)
            self.iam_client = boto3.client('iam',config=configuration)
        except Exception as ex:
            logger.error("Error occurred while initializing glue and iam client objects: %s", str(ex))
            raise ex


    def init_connections(self):
        configuration=helper.get_configuration()
        self.catalog_id = boto3.client('sts',config=configuration).get_caller_identity().get('Account')
        self.glue_results = []
        self.jobs={'Jobs':[]}
        self.crawlers={'Crawlers':[]}
        paginator_get_jobs=self.glue_client.get_paginator('get_jobs')
        for page in paginator_get_jobs.paginate():
            self.jobs['Jobs']+=page['Jobs']
        paginator_get_crawlers=self.glue_client.get_paginator('get_crawlers')
        for page in paginator_get_crawlers.paginate():
            self.crawlers['Crawlers']+= page['Crawlers']
        self.resource_list = []
    def list_resources(self):
        """
        Returns a list of all resources associated with the Glue client.
        """
        try:
            self.init_connections()
            self.resource_list.append(self.catalog_id)
            for job in self.jobs['Jobs']:
                self.resource_list.append(job['Name'])
            for crawler in self.crawlers['Crawlers']:
                self.resource_list.append(crawler['Name'])
            return self.resource_list

        except botocore.exceptions.ClientError as ex:
            logger.error("Error occurred when listing Glue resources: %s", str(ex))
            raise


    def check_data_catalog_encryption(self, catalog_id):
        """
        Provides the encryption settings of the provided Catalog_id.

        Args:
            CatalogID (str): .

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        Raises:
            Exception: If any error occurs during the check.

        """
        try:
            self.init_connections()
            if self.catalog_id == catalog_id:
                logger.info("gathering the datacatalog encryption settings for %s", catalog_id)
                try:
                    response = self.glue_client.get_data_catalog_encryption_settings(CatalogId=self.catalog_id)

                    if response['DataCatalogEncryptionSettings']['EncryptionAtRest']['CatalogEncryptionMode'] != "DISABLED":
                        check_result = application_constants.ResultStatus.PASSED
                    else:
                        check_result = application_constants.ResultStatus.FAILED

                except self.glue_client.exceptions.ClientError as ex:
                    error_code = ex.response['Error']['Code']
                    if error_code == 'ServerSideEncryptionConfigurationNotFoundError':
                        check_result = application_constants.ResultStatus.FAILED
                    else:
                        logger.error("error while gathering the datacatalog encryption settings for %s: %s", catalog_id, str(ex))
                        check_result = application_constants.ResultStatus.UNKNOWN
                except Exception as ex:
                    logger.error("gathering the datacatalog encryption settings for %s: %s", catalog_id, str(ex))
                    raise ex

                logger.info("Completed fetching datacatalog encryption settings for catalog ID %s", catalog_id)
                return check_result
        except botocore.exceptions.ClientError as ex:
            logger.error("Error occurred when checking datacatalog encryption for Glue resources: %s", str(ex))
            raise

    def check_connection_passwords(self, catalog_id):
        """
        fetches the data about the glue connection for the provided Catalog_id.

        Args:
            CatalogID (str): .

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        Raises:
            Exception: If any error occurs during the check.

        """
        try:
            logger.info("fetching the glue connection data for %s", catalog_id)
            self.init_connections()
            if self.catalog_id == catalog_id:
                try:
                    response = self.glue_client.get_data_catalog_encryption_settings(CatalogId=catalog_id)
                    if response['DataCatalogEncryptionSettings']['ConnectionPasswordEncryption']['ReturnConnectionPasswordEncrypted'] == True:
                        check_result = application_constants.ResultStatus.PASSED
                    else:
                        check_result = application_constants.ResultStatus.FAILED
                except self.glue_client.exceptions.ClientError as ex:
                    logger.error("error while fetching the glue connection data for %s: %s", catalog_id, str(ex))
                    check_result = application_constants.ResultStatus.UNKNOWN
                except Exception as ex:
                    logger.error("exception while fetching the glue connection data for %s: %s", catalog_id, str(ex))
                    raise ex

                logger.info("Completed fetching Glue connection data for catalog ID %s", catalog_id)
                return check_result
        except botocore.exceptions.ClientError as ex:
            logger.error("Error occurred when checking glue connection data: %s", str(ex))
            raise


    def check_encryption_at_rest(self, resource_name):
        """
        Checks if encryption at rest is enabled for a given Glue job or crawler.

        Args:
            resource_name (str): The name of the Glue job or crawler to check.

        Returns:
            ResultStatus: A `ResultStatus` enum value indicating whether the check passed or failed.

        Raises:
            Exception: If any error occurs during the check.
        """
        logger.info("Started checking for encryption at rest for resource: %s", resource_name)
        try:
            self.init_connections()

            for job in self.jobs['Jobs']:
                if job['Name'] != resource_name:
                    continue

                default_args = job["DefaultArguments"]
                security_config_name = job.get("SecurityConfiguration")

                if "--encryption-type" in default_args:
                    check_result = application_constants.ResultStatus.PASSED
                elif security_config_name:
                    security_config_info = self.glue_client.get_security_configuration(Name=security_config_name)
                    s3_encryption_mode = security_config_info['SecurityConfiguration']['EncryptionConfiguration']['S3Encryption'][0]['S3EncryptionMode']
                    check_result = application_constants.ResultStatus.PASSED if s3_encryption_mode else application_constants.ResultStatus.FAILED
                else:
                    check_result = application_constants.ResultStatus.FAILED
                return check_result

            for crawler in self.crawlers['Crawlers']:
                if crawler['Name'] != resource_name:
                    continue

                if "CrawlerSecurityConfiguration" in crawler:
                    security_config_name = crawler["CrawlerSecurityConfiguration"]
                    security_config_info = self.glue_client.get_security_configuration(Name=security_config_name)
                    s3_encryption_mode = security_config_info['SecurityConfiguration']['EncryptionConfiguration']['S3Encryption'][0]['S3EncryptionMode']
                    check_result = application_constants.ResultStatus.PASSED if s3_encryption_mode else application_constants.ResultStatus.FAILED
                else:
                    check_result = application_constants.ResultStatus.FAILED
                return check_result

        except Exception as ex:
            logger.error("Error occurred when checking encryption of rest for Glue resources: %s", str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN
            return check_result
        finally:
            logger.info("Finished checking for encryption at rest for resource: %s", resource_name)


    def check_iam_permissions(self, resource_name):
        """
        checks if the job has necessary IAM permissions to perform actions in glue.

        Args:
            iam_role (str): AWS IAM role name .

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.

        Raises:
            Exception: If any error occurs during the check.

        """
        try:
            self.init_connections()
            logger.info("checking permissions for the iam role %s", resource_name)
            for job in self.jobs['Jobs']:
                    if job['Name'] == resource_name:
                        try:
                            role_name = job['Role']
                            role_name = role_name.split("/")[-1]
                            response = self.iam_client.get_role(RoleName=role_name)
                            if response:
                                check_result = application_constants.ResultStatus.PASSED
                            else:
                                check_result = application_constants.ResultStatus.FAILED


                        except self.glue_client.exceptions.ClientError as ex:
                            logger.error("error while getting iam role for %s: %s",resource_name, str(ex))
                            check_result = application_constants.ResultStatus.UNKNOWN
                        except Exception as ex:
                            logger.error("exception while checking for role permissions of  %s: %s", resource_name, str(ex))
                            check_result = application_constants.ResultStatus.UNKNOWN
                            raise ex

                        logger.info("Completed checking the role permissions of %s", resource_name)
                        return check_result
            for crawler in self.crawlers['Crawlers']:
                    if crawler['Name'] == resource_name:
                        try:
                            role_name = crawler['Role']
                            role_name = role_name.split("/")[-1]
                            response = self.iam_client.get_role(RoleName=role_name)
                            if response:
                                check_result = application_constants.ResultStatus.PASSED
                            else:
                                check_result = application_constants.ResultStatus.FAILED


                        except self.glue_client.exceptions.ClientError as ex:
                            logger.error("error while getting iam role for %s: %s",resource_name, str(ex))
                            check_result = application_constants.ResultStatus.UNKNOWN
                        except Exception as ex:
                            logger.error("exception while checking for role permissions of  %s: %s", resource_name, str(ex))
                            raise ex

                        logger.info("Completed checking the role permissions of %s", resource_name)
                        return check_result
        except botocore.exceptions.ClientError as ex:
            logger.error("Error occurred when checking IAM Permissions for Glue resources: %s", str(ex))
            raise

    def check_glue_tags(self, resource_name, required_tags=None):
        """
        Checks if the specified Glue Operation has the required tags.

        Args:
            glue_arn (str): The arn of the Glue operation.

        Returns:
            None: This method updates the status of the check in the `self.service` dictionary.
        """
        try:
            self.init_connections()
            self.jobs = self.glue_client.get_jobs()
            self.crawlers = self.glue_client.get_crawlers()

            if required_tags is None:
                required_tags = application_constants.Generic.REQUIRED_TAGS

            if self.catalog_id == resource_name:
                return None

            if not self.jobs['Jobs'] and not self.crawlers['Crawlers']:
                check_result = application_constants.ResultStatus.UNKNOWN
            else:
                for job in self.jobs['Jobs']:
                    if job['Name'] == resource_name:
                        resource_arn = 'arn:aws:glue:{}:{}:{}/{}'.format(self.glue_client.meta.region_name, self.catalog_id, constants.resource_arns['Job'], resource_name)
                        check_result = self.fetch_tags(required_tags, resource_arn)
                        return check_result
                else:
                    check_result = application_constants.ResultStatus.UNKNOWN

                for crawler in self.crawlers['Crawlers']:
                    if crawler['Name'] == resource_name:
                        resource_arn = 'arn:aws:glue:{}:{}:{}/{}'.format(self.glue_client.meta.region_name, self.catalog_id, constants.resource_arns['Crawler'], resource_name)
                        check_result = self.fetch_tags(required_tags, resource_arn)
                        return check_result
                else:
                    check_result = application_constants.ResultStatus.UNKNOWN

        except Exception as ex:
            logger.error("Error occurred when checking tags for Glue resources: %s", str(ex))
            check_result = application_constants.ResultStatus.UNKNOWN

        return check_result

    def fetch_tags(self, required_tags, resource_arn, missing_tags=[]):
        try:
            if not required_tags:
                check_result = application_constants.ResultStatus.PASSED
            else:
                logger.info("Checking the tags for glue_operation %s", resource_arn)
                tags = self.glue_client.get_tags(
                    ResourceArn=resource_arn
                )
                missing_tags = [tag for tag in required_tags if tag.split(":")[0] not in [t for t in tags['Tags']]]
                check_result = application_constants.ResultStatus.PASSED if not any(missing_tags) else application_constants.ResultStatus.FAILED

            logger.info("Tags check for Glue %s completed successfully.", resource_arn)
            return check_result
        except botocore.exceptions.ClientError as ex:
            if ex.response['Error']['Code'] == 'NoSuchTagSet':
                check_result = application_constants.ResultStatus.FAILED
            else:
                logger.error(f"An error occurred while checking the tags for Glue {resource_arn}: {str(ex)}")
                check_result = application_constants.ResultStatus.UNKNOWN

            logger.info(f"Completed checking the tags")
            return check_result
        except Exception as ex:
            logger.exception(f"An error occurred while checking the tags for GLUE {resource_arn}: {str(ex)}")
            check_result = application_constants.ResultStatus.UNKNOWN

            logger.info(f"Completed checking the tags for glue arn : {resource_arn}")
            return check_result
